/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 2014 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */

//
// This file provides common types for both UVM driver and RM's UVM interface.
//

#ifndef _NV_UVM_TYPES_H_
#define _NV_UVM_TYPES_H_

#include "nvtypes.h"
#include "nvgputypes.h"

#define UVM_UUID_LEN 16
typedef struct UvmGpuUuid_tag
{
    NvU8 uuid[UVM_UUID_LEN];
} UvmGpuUuid;

typedef unsigned long long UvmGpuPointer;

//
// The following typdef's serve to explain the resources they point to.
// The actual resources remain RM internal and not exposed.
//
typedef void *uvmGpuSessionHandle;              // gpuSessionHandle
typedef void *uvmGpuAddressSpaceHandle;         // gpuAddressSpaceHandle
typedef void *uvmGpuChannelHandle;              // gpuChannelHandle
typedef void *uvmGpuCopyEngineHandle;           // gpuObjectHandle

typedef struct UvmGpuChannelPointers_tag
{
    volatile unsigned *GPGet;
    volatile unsigned *GPPut;
    UvmGpuPointer     *gpFifoEntries;
    unsigned           numGpFifoEntries;
    unsigned           channelClassNum;
    NvNotification    *errorNotifier;
    NvBool            *eccErrorNotifier;
} UvmGpuChannelPointers;

typedef struct UvmGpuCaps_tag
{
    unsigned largePageSize;
    unsigned smallPageSize;
    unsigned eccMask;
    unsigned eccOffset;
    void    *eccReadLocation;
    unsigned pcieSpeed;
    unsigned pcieWidth;
    NvBool   bEccEnabled;
} UvmGpuCaps;

/*******************************************************************************
    uvmEventStartDevice
    This function will be called by the GPU driver once it has finished its
    initialization to tell the UVM driver that this GPU has come up.
*/
typedef RM_STATUS (*uvmEventStartDevice_t) (UvmGpuUuid *pGpuUuidStruct);

/*******************************************************************************
    uvmEventStopDevice
    This function will be called by the GPU driver to let UVM know that a GPU
    is going down.
*/
typedef RM_STATUS (*uvmEventStopDevice_t) (UvmGpuUuid *pGpuUuidStruct);

/*******************************************************************************
    uvmEventIsrTopHalf_t
    This function will be called by the GPU driver to let UVM know
    that an interrupt has occurred.

    Returns:
        RM_OK if the UVM driver handled the interrupt
        RM_ERR_NO_INTR_PENDING if the interrupt is not for the UVM driver
*/
#if defined (__linux__)
typedef RM_STATUS (*uvmEventIsrTopHalf_t) (void);
#else
typedef void (*uvmEventIsrTopHalf_t) (void);
#endif

struct UvmOpsUvmEvents
{
    uvmEventStartDevice_t startDevice;
    uvmEventStopDevice_t  stopDevice;
    uvmEventIsrTopHalf_t  isrTopHalf;
};

#endif // _NV_UVM_TYPES_H_
